from pysmash import api, utils, exceptions

BRACKET_URL = '/phase_group/'
VALID_BRACKET_PARAMS = ['sets', 'entrants']


def players(bracket_id, filter_response=True):
    uri = BRACKET_URL + str(bracket_id)

    response = api.get(uri, VALID_BRACKET_PARAMS)

    if filter_response:
        response = _filter_player_response(response)

    return response


def sets(bracket_id, filter_response=True):
    uri = BRACKET_URL + str(bracket_id)

    response = api.get(uri, VALID_BRACKET_PARAMS)

    if filter_response:
        response = _filter_set_response(response)

    return response


def sets_played_by_player(bracket_id, tag):
    try:
        tag = str(tag)
        tag = tag.lower()
    except:
        msg = "Given player tag is not and cannot be converted into a string"
        raise exceptions.ValidationError(msg)

    uri = BRACKET_URL + str(bracket_id)

    response = api.get(uri, VALID_BRACKET_PARAMS)
    return _filter_sets_given_player(response, tag)


def _filter_sets_given_player(response, tag):

    result_player = None
    players = _filter_player_response(response)
    for p in players:
        if p['tag'].lower() == tag:
            result_player = p

    if result_player is None:
        return []

    player_sets = []
    sets = _filter_set_response(response)

    # grab sets the player was involved in
    for _set in sets:
        # ignore bye sets
        if _set['entrant_1_id'] == 'None' or _set['entrant_2_id'] == 'None':
            continue

        # winner's id of `None` means the set was not played
        # this happens when both players made it out of pools, etc
        if _set['winner_id'] == 'None':
            continue

        player_is_entrant1 = str(result_player['entrant_id']) == _set['entrant_1_id']
        player_is_entrant2 = str(result_player['entrant_id']) == _set['entrant_2_id']
        if player_is_entrant1 or player_is_entrant2:
            _set['player_id'] = result_player['entrant_id']
            if player_is_entrant1:
                _set['opponent_id'] = int(_set['entrant_2_id'])
            elif player_is_entrant2:
                _set['opponent_id'] = int(_set['entrant_1_id'])
            player_sets.append(_set)

    # grab information about player's opponents
    for item in player_sets:
        for opponent in players:
            if item['opponent_id'] == opponent['entrant_id']:
                item['opponent_info'] = opponent

    return {
        'player': result_player,
        'sets': player_sets
    }


def _filter_player_response(response):
    players = []
    entrants = response['entities']['entrants']

    for entrant in entrants:
        player = _get_player_from_entrant(entrant)
        players.append(player)

    return players


def _filter_set_response(response):
    entities = response.get('entities', None)

    if entities is None:
        return []

    bracket_sets = response['entities'].get('sets', None)
    if bracket_sets is None:
        return []

    groups = entities.get('groups', None)
    if groups is None:
        return []
    is_final_bracket = _is_final_bracket(groups)

    results_sets = []
    for bracket_set in bracket_sets:
        # don't return `projected` brackets
        if 'preview' in str((bracket_set['id'])):
            break

        _set = _get_set_from_bracket(bracket_set, is_final_bracket)
        results_sets.append(_set)

    return results_sets


def _is_final_bracket(groups):
    is_final_bracket = False
    w_id = groups.get('winnersTargetPhaseId', None)
    if w_id == 'None' or w_id is None:
        is_final_bracket = True
    return is_final_bracket


def _get_set_from_bracket(bracket_set, is_final_bracket):
    return {
        'id': str(bracket_set['id']),  # make all IDS ints?
        'entrant_1_id': str(bracket_set['entrant1Id']),
        'entrant_2_id': str(bracket_set['entrant2Id']),
        'entrant_1_score': bracket_set['entrant1Score'],
        'entrant_2_score': bracket_set['entrant2Score'],
        'winner_id': str(bracket_set['winnerId']),
        'loser_id': str(bracket_set['loserId']),
        'full_round_text': bracket_set['fullRoundText'] if is_final_bracket else 'pools',
        'medium_round_text': bracket_set['midRoundText'] if is_final_bracket else 'pools',
        'short_round_text': bracket_set['shortRoundText'] if is_final_bracket else 'pools',
        'bracket_id': str(bracket_set['phaseGroupId'])
    }


def _get_player_from_entrant(entrant):
    participant_id = str(entrant['participantIds'][0])
    player_id = str(entrant['playerIds'][participant_id])
    player_dict = entrant['mutations']['players'][player_id]
    participant_dict = entrant['mutations']['participants'][participant_id]

    return {
        'entrant_id': entrant['id'],
        'tag': player_dict['gamerTag'],
        'fname': utils.get_subfield(participant_dict, 'contactInfo', 'nameFirst'),
        'lname': utils.get_subfield(participant_dict, 'contactInfo', 'nameLast'),
        'state': player_dict['state'],
        'country': player_dict['country'],
        'final_placement': entrant['finalPlacement'],
        'seed': entrant['initialSeedNum']
    }



from pysmash import exceptions
import requests


SMASHGG_API_URL = 'api.smash.gg'


def get(uri, params=[]):
    """"Performs a get request and return the contents of the response"""
    params = _prepare_params(params)
    url = "https://%s/%s" % (SMASHGG_API_URL, uri + params)

    r = requests.get(url)

    data = r.json()
    if r.status_code == 200:
        return data
    raise exceptions.ResponseError(data['message'], r.status_code)


def _prepare_params(params):
    """return params as SmashGG friendly query string"""
    query_string = ''

    if len(params) == 0:
        return query_string

    prefix = '?expand[]='
    query_string = prefix + '&expand[]='.join(params)

    return query_string



from distutils.core import setup
import pysmash


setup(name="pysmash",
      author="Peter Wensel",
      url="https://github.com/PeterCat12/pysmash",
      version=pysmash.__version__,
      packages=[
          'pysmash',
      ],
      install_requires=[
          'requests==2.10.0',
      ]
)



from pysmash import api, brackets, utils

TOURNAMENT_PREFIX = 'tournament/'
EVENT_URL = '/event/'

VALID_PARAMS = ['event', 'phase', 'groups', 'stations']


def show(tournament_name, params=[], filter_response=True):
    """Retrieve a single tournament record by `tournament name`"""
    utils._validate_query_params(params=params, valid_params=VALID_PARAMS, route_type='tournament')
    uri = TOURNAMENT_PREFIX + tournament_name

    response = api.get(uri, params)

    if filter_response:
        response = _filter_tournament_response(response, params)

    return response


def show_events(tournament_name):
    """Returns a list of events for a tournament"""
    uri = TOURNAMENT_PREFIX + tournament_name

    response = api.get(uri, ['event'])

    result = {}
    result = _append_events(response, result)
    return result


def show_with_brackets(tournament_name, event, tournament_params=[], ):
    """Returns tournament meta information along with a list of bracketIds for an event"""
    tournament = show(tournament_name, tournament_params)
    brackets = event_brackets(tournament_name, event)

    return utils.merge_two_dicts(tournament, brackets)


def show_sets(tournament_name, event, tournament_params=[]):
    """Returns all sets from a tournament"""
    tournament = show_with_brackets(tournament_name, event, tournament_params)

    results = []
    for bracket_id in tournament['bracket_ids']:
        bracket_sets = brackets.sets(bracket_id)
        for _set in bracket_sets:
            if len(_set) > 0:
                results.append(_set)

    return results


def show_players(tournament_name, event_name, tournament_params=[]):
    """Returns all players from a tournament"""
    tournament = show_with_brackets(tournament_name, tournament_params)

    results = []
    for bracket_id in tournament['bracket_ids']:
        bracket_players = brackets.players(bracket_id)
        for player in bracket_players:
            results.append(player)
    return list({v['tag']: v for v in results}.values())


def show_player_sets(tournament_name, event, player_tag):
    """Returns all players from a tournament"""
    tournament = show_with_brackets(tournament_name, event)

    player = None
    bracket_sets = []

    for bracket_id in tournament['bracket_ids']:
        player_sets = brackets.sets_played_by_player(bracket_id, player_tag)
        if len(player_sets) == 0:
            continue

        if player is None:
            player = player_sets['player']

        _sets = player_sets['sets']
        bracket_sets = bracket_sets + _sets

    return {
        'player': player,
        'sets': bracket_sets
    }


def event_brackets(tournament_name, event='wii-u-singles', filter_response=True):
    # first, get the events for the tournament...
    events = show_events(tournament_name)

    """Returns a list of brackets ids for an event"""
    utils._validate_query_params(params=[event], valid_params=events['events'], route_type='event')

    uri = TOURNAMENT_PREFIX + tournament_name + '/event/' + event

    response = api.get(uri, ['groups'])

    if filter_response:
        response = _filter_event_bracket_response(response)

    return response


def _filter_event_bracket_response(response):
    """Filters the Smash.gg response to something more managable"""
    bracket_ids = []
    for bracket in response['entities']['groups']:
        bracket_ids.append(str(bracket['id']))

    return {
        'bracket_ids': bracket_ids,
        'event_name': response['entities']['event']['typeDisplayStr'],
        'bracket_full_source_url': response['entities']['event']['slug']
    }


def _filter_tournament_response(response, params=[]):
    """Filters the Smash.gg response to something more managable"""
    result = {
        'tournament_id': response['entities']['tournament']['id'],
        'venue_name': response['entities']['tournament']['venueName'],
        'venue_addresss': response['entities']['tournament']['venueAddress'],
        'name': response['entities']['tournament']['name'],
        'tournament_full_source_url': response['entities']['tournament']['slug'],
        'links': response['entities']['tournament']['links'],
        'state_short': response['entities']['tournament']['regionDisplayName'],
        'details': response['entities']['tournament']['details']
    }

    if 'event' in params:
        result = _append_events(response, result)
    if 'phase' in params:
        result = _append_phases(response, result)
    if 'groups' in params:
        result = _append_groups(response, result)
    return result


def _append_groups(response, result):
    result['groups'] = []
    groups = response['entities'].get('groups', [])
    for group in groups:
        group_dict = {
            "group_id": group['id'],
            'phase_id': group['phaseId'],
            'title': group['title'],
            'winners_target_phase': group['winnersTargetPhaseId'],
        }
        result['groups'].append(group_dict)
    return result


def _append_phases(response, result):
        result['phases'] = []
        phases = response['entities'].get('phase', [])
        for phase in phases:
            phase_dict = {
                "phase_id": phase['id'],
                'event_id': phase['eventId'],
                'phase_name': phase['name'],
                'is_exhibition': phase['isExhibition'],
                'type_id': phase['typeId']
            }
            result['phases'].append(phase_dict)
        return result


def _append_events(response, result):
        result['events'] = []
        events = response['entities'].get('event', [])
        for event in events:
            slug = event['slug']
            slug = slug.split("/")
            result['events'].append(slug[-1])
        return result



class ExceptionWithStatusCode(Exception):
    def __init__(self, message, status_code):

        # Call the base class constructor with the parameters it needs
        super(ExceptionWithStatusCode, self).__init__(message)
        self.status_code = status_code


class ValidationError(ExceptionWithStatusCode):
    def __init__(self, message, status_code=400):
        super(ValidationError, self).__init__(message, status_code=status_code)


class ResponseError(ExceptionWithStatusCode):
    def __init__(self, message, status_code):
        super(ResponseError, self).__init__(message=message, status_code=status_code)
